#ifndef LIBHARKIO_DATA_H
#define LIBHARKIO_DATA_H

#include <iostream>
#include <sstream>
#include <fstream>
#include "chunk.h"
#include "common.h"

namespace harkio {

/**********************************************************************/
//! Data抽象基底クラス
/*!
 * 	Data階層に属する実際のデータは、このクラスの派生として実現する
 */
/**********************************************************************/
class Data {
	friend class RIFF;
protected:
	//! DataへChunkを追加する前に、エラーチェックを行う
	virtual void addChunkCheck(Chunk *chunk) = 0;

	//! 派生クラスの識別子を取得する
	virtual const char *getIdentifierFromBase() = 0;

	//! 保持しているChunkの配列
	std::vector<Chunk*> chunk;

	//! サイズヘッダーのサイズを含む自身のクラスのサイズを取得する関数。サイズフィールドはないが、親の階層がsetSizeする際に必要になる
	int getClassSize();

	//! サイズフィールドの更新を行う
	/*! Dataクラスには、サイズフィールドは無いが、別の階層から再帰的に呼び出されるので、必要になる
	 */
	void setSize();

public:
	Data() {}
	virtual ~Data() {
		for (unsigned i = 0; i < chunk.size(); i++) {
			delete chunk[i];
		}
	}

	//! 書き込む前にデータが揃っているか確認する
	/*! サイズ以外のデータの整合性をチェックする(再帰的に呼び出している)
	 */
	virtual bool isComplete()= 0;

	//! DataへChunkの追加を行う
	virtual void addChunk(Chunk *);

	//! Dataへ追加されているChunkのリストを取得する。派生クラスへのキャストは無い
	virtual std::vector<Chunk*> getChunks();

	//! Dataが保持しているChunkの取得を行う。派生クラスへのキャスト有り
	/*
	 * \param name テンプレート引数へ、キャストする派生クラスの型を指定
	 */
	template<typename T> T* getChunk() {
		for(unsigned int i=0; i < chunk.size(); i++) {
		if(chunk[i]->getIdentifierFromBase() == T::getIdentifier()) {
				T* tmp = dynamic_cast<T*>(chunk[i]);
				return tmp;
			}
		}
		return NULL;
	}
	//! ファイルストリームからこのクラスのデータを読みとる
	virtual  void readFile(std::ifstream &fs) = 0;

	//! ファイルストリームへこのクラスのデータを書き込む
	virtual  void writeFile(std::ofstream &fs) = 0;

};

/**********************************************************************/
//! HGTFクラス
/*!
 * 	HGTF(HondaGiken TransferFunction)データクラス
 */
/**********************************************************************/
class HGTF : public Data {
	friend class HARK;
private:
	//! 書き込む前にデータが揃っているか確認する
	bool isComplete();

	//! DataへChunkを追加する前に、エラーチェックを行う
	void addChunkCheck(Chunk *chunk);

	//! ファイルストリームからこのクラスのデータを読みとる
	void readFile(std::ifstream &fs);

	//! ファイルストリームへこのクラスのデータを書き込む
	void writeFile(std::ofstream &fs);

	//! 基底クラスからこのクラスの識別子を取得する
	const char *getIdentifierFromBase() {
		return getIdentifier();
	}

	//! このクラスの識別子を取得する
	static const char *getIdentifier() {
		// 識別子
		static const char *identifier=NULL;
		if(!identifier) {
			identifier ="HGTF";
		}
		return identifier;
	}
public:
	HGTF() {
	}
	~HGTF() {}
};

} // namespace

#endif
